/*
 * Simple program for enforcing clipboard preservation.
 * This simple utility is coded mainly for being able to have
 * exit-persistent clipboards (that is to not lose clipboard
 * content on application end) in Tcl/Tk applications
 * at Linux-like systems.
 * It was tested under XFCE Ubuntu 16.04 environment.
 *
 * This is free software, set into public domain.
 * THERE IS ABSOLUTELY NO (NEITHER IMPLIED) WARRANTY.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

int clipboard_form;

static void enforce_clipboard (GtkApplication *app, gpointer user_data)
{
  // dealing with clipboard only; it could be set to:
  // GDK_SELECTION_PRIMARY
  // GDK_SELECTION_SECONDARY
  // GDK_SELECTION_CLIPBOARD
  GdkAtom atom = GDK_SELECTION_CLIPBOARD;
  // taking the (respective) clipboard
  GtkClipboard *clipboard = gtk_clipboard_get(atom);

  // clipboard data holders
  gchar *clipboard_text = NULL;
  GdkPixbuf *clipboard_image = NULL;

  // reading the clipboard data (set by an outer process) that has to be stored,
  // and putting it into the clipboard locally (if the data are not not empty)
  if (clipboard_form==0) {
    clipboard_text = gtk_clipboard_wait_for_text(clipboard);
    if (clipboard_text != NULL) {
      gtk_clipboard_set_text(clipboard, clipboard_text, -1);
    }
  } else {
    clipboard_image = gtk_clipboard_wait_for_image(clipboard);
    if (clipboard_image != NULL) {
      gtk_clipboard_set_image(clipboard, clipboard_image);
    }
  }

  // storing the clipboard content so that it survives the exits
  gtk_clipboard_store(clipboard);
  // not calling g_object_unref on clipboard as it would be "prematurely finalized"

  // g_free checks whether it is NULL, thus do not need to check it too
  g_free(clipboard_text);
  clipboard_text = NULL;
  // g_clear_object sets the respective pointer to NULL too
  if (clipboard_image != NULL) {
    g_clear_object(&clipboard_image);
  }
  // should not be necessary to deal with cleaning stuff here, since it ends anyway;

  // app would went to ending here even without this asking to end the app
  g_application_quit(G_APPLICATION(app));
}

int main (int argc, char **argv)
{
  GtkApplication *app;
  int status;

  if ((argc==2) && (strcmp(argv[1],"--help")==0)) {
    fprintf(stdout, "utility for enforcing clipboard preservation at GTK-based environments\n");
    fprintf(stdout, "supported forms: \"text\" (default) and \"image\", set by respective parameters\n");
    return 0;
  }

  // reading what is the requested selection type
  if (argc==1) {
    clipboard_form = 0;
  } else if (argc==2) {
    if ((strcmp(argv[1],"text")==0) || (strcmp(argv[1],"--text")==0)) {
      clipboard_form = 0;
    } else if ((strcmp(argv[1],"image")==0) || (strcmp(argv[1],"--image")==0)) {
      clipboard_form = 1;
    } else {
      fprintf(stderr, "unknown format specifier, should be: \"[--]text\", \"[--]image\"\n");
      fprintf(stderr, "for text (default) or image data content in clipboard\n");
      return 1;
    }
  } else {
    fprintf(stderr, "at most one parameter (for specifying the data form) is admissible\n");
    return 1;
  }

  // to avoid warning messages (about 'accessibility bus') at systems without complete GNOME support
  g_setenv("NO_AT_BRIDGE", "1", TRUE);

  // creating the application, not dealing with single-instance stuff
  app = gtk_application_new("clipboard.storage.enforcer", G_APPLICATION_NON_UNIQUE);
  // when the application is activated, it calls the function for clipboard management
  g_signal_connect(app, "activate", G_CALLBACK(enforce_clipboard), NULL);
  // starting the application (it creates the resources needed during clipboard management),
  // and the "activate" signal is sent within that process;
  // not dealing with command-line arguments internally, thus set to 0/NULL here
  status = g_application_run(G_APPLICATION(app), 0, NULL);
  // since neither window nor a loop got created, it ends after the clipboard dealing
  // (even if the explicit request for ending the app were not there)
  g_object_unref(app);
  return status;
}


